// Copyright © 2005, Esko Luontola <www.orfjackal.net>
// This software is released under the Apache License 2.0.
// The license text is at http://www.apache.org/licenses/LICENSE-2.0

package net.orfjackal.sorsastus;

import org.jetbrains.annotations.NotNull;

import java.math.BigInteger;
import java.util.*;

/**
 * Collects the profiler log statistics of the methods of one class.
 */
public class ProfilerClassStatistics {

    @NotNull
    private String classID;

    @NotNull
    private Map<String, ProfilerMethodStatistics> methods;

    public ProfilerClassStatistics(String classID) {
        if (classID == null) {
            throw new NullPointerException();
        }
        this.classID = classID;
        this.methods = new HashMap<String, ProfilerMethodStatistics>();
    }

    @NotNull
    public ProfilerMethodStatistics getMethodStats(String methodID) {
        if (methodID == null) {
            throw new NullPointerException();
        }
        ProfilerMethodStatistics methodStats = methods.get(methodID);
        if (methodStats == null) {
            methodStats = new ProfilerMethodStatistics(classID, methodID);
            methods.put(methodID, methodStats);
        }
        return methodStats;
    }

    @NotNull
    public Map<String, ProfilerMethodStatistics> getMethods() {
        return methods;
    }

    @NotNull
    public BigInteger getTotalExecutionTime() {
        BigInteger execTime = BigInteger.ZERO;
        for (ProfilerMethodStatistics methodStats : methods.values()) {
            execTime = execTime.add(methodStats.getExecutionTime());
        }
        return execTime;
    }
}
