// Tietorakenteiden harjoitusty (syksy 2004), Esko Luontola
// Tietoliikenneverkon reititysalgoritmien vertailu, "Sim Router"

/**
 * Pakettipuskuri kytettvksi reitittimien lhetys- ja vastaanottopuskureissa.
 */
public class PacketBuffer {
	
	/**
	 * Puskuri.
	 */
	private final Packet[] buffer;
	
	/**
	 * Ensimminen kytetty paikka puskurissa.
	 */
	private int front;
	
	/**
	 * Seuraava vapaa paikka puskurissa.
	 */
	private int back;
	
	/**
	 * Puskurin kytetyiden alkioiden lukumr.
	 */
	private int count;
	
	/**
	 * Luo halutun kokoisen puskurin. Jos size on pienempi kuin 1, kytetn 
	 * oletusarvoa 10.
	 */
	public PacketBuffer(int size) {
		if (size < 1) {
			size = 10;
		}
		buffer = new Packet[size];
		front = 0;
		back = 0;
		count = 0;
	}
	
	/**
	 * Laittaa paketin tmn puskurin viimeiseksi. Palauttaa onnistuessaan true,
	 * tai false jos puskuri oli tynn, jolloin pakettia ei list puskuriin. 
	 * Jos packet on null, sit ei list puskuriin.
	 */
	public boolean push(Packet packet) {
		if (isFull() || packet == null) {
			return false;
		} else {
			buffer[back] = packet;
			back = (back + 1) % buffer.length;
			count++;
			return true;
		}
	}
	
	/**
	 * Palauttaa puskurissa ensimmisen olevan alkion ja poistaa sen puskurista.
	 * Jos puskuri on tyhj, niin palautetaan null.
	 */
	public Packet pop() {
		if (isEmpty()) {
			return null;
		} else {
			Packet packet = buffer[front];
			buffer[front] = null;
			front = (front + 1) % buffer.length;
			count--;
			return packet;
		}
	}
	
	/**
	 * Palauttaa puskurissa ensimmisen olevan alkion poistamatta sit puskurista.
	 * Jos puskuri on tyhj, niin palauttaa null.
	 */
	public Packet getFirst() {
		return buffer[front];
	}
	
	/**
	 * Puskurissa olevien pakettien lukumr.
	 */
	public int getCount() {
		return count;
	}
	
	/**
	 * Palauttaa true, jos puskuri on tynn, muutoin false.
	 */
	public boolean isFull() {
		return (count == buffer.length);
	}
	
	/**
	 * Palauttaa true, jos puskuri on tyhj, muutoin false.
	 */
	public boolean isEmpty() {
		return (count == 0);
	}
	
	/**
	 * Palauttaa puskurin tilan merkkiesityksen. Datavirta on vasemmalta oikealle.
	 */
	public String toString() {
		StringBuffer s = new StringBuffer(buffer.length+2);
		for (int i = buffer.length-1; i >= 0; i--) {
			if (buffer[i] == null) {
				s.append('-');
			} else {
				s.append('+');
			}
			if (i == back) {
				s.append('>');
			}
			if (i == front) {
				s.append('>');
			}
		}
		return s.toString();
	}
	
	public static void main(String[] args){
		
		PacketBuffer buffer = new PacketBuffer(10);
		
		System.out.println(buffer);
		
		for (int i = 0; i < 12; i++) {
			buffer.push(new Packet(0,0,0,0));
			System.out.println(buffer);
		}
		for (int i = 0; i < 12; i++) {
			buffer.pop();
			System.out.println(buffer);
		}
		
		buffer.push(new Packet(0,0,0,0));
		System.out.println(buffer);
		buffer.push(new Packet(0,0,0,0));
		System.out.println(buffer);
		
		buffer.pop();
		System.out.println(buffer);
		buffer.push(new Packet(0,0,0,0));
		System.out.println(buffer);
		buffer.pop();
		System.out.println(buffer);
		buffer.push(new Packet(0,0,0,0));
		System.out.println(buffer);
		
		buffer.pop();
		System.out.println(buffer);
		buffer.pop();
		System.out.println(buffer);
		
	}
}
